/* --------------------------------------------------------------------------
 *
 * License
 *
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License.  You
 * may obtain a copy of the License at http://www.jabber.com/license/ or at
 * http://www.opensource.org/.  
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are 
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 *
 * Portions Copyright (c) 1998-1999 Jeremie Miller.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 * 
 * --------------------------------------------------------------------------*/

#include "conference.h"

taffil affiliation_level(cnr room, jid user)
{
    if(is_owner(room, user))
    {
        return TAFFIL_OWNER;
    }
    else if(is_admin(room, user))
    {
        return TAFFIL_ADMIN;
    }           
    else if(is_member(room, user))
    {           
        return TAFFIL_MEMBER;
    }   
    else if(is_outcast(room, user))
    {       
        return TAFFIL_OUTCAST;
    }   

    return TAFFIL_NONE;
}           

trole role_level(cnr room, jid user)
{
    if(is_leaving(room, user))
    {
	return TROLE_NONE;
    }
    else if(is_moderator(room, user))
    {   
        return TROLE_MODERATOR;
    }       
    else if(is_participant(room, user))
    {
        return TROLE_PARTICIPANT;
    }   
    else if(is_visitor(room, user))
    {       
        return TROLE_VISITOR;
    }   
	        
    return TROLE_NONE;
}   

int add_affiliate(xht hash, jid userid, xmlnode details)
{
    pool p;
    xmlnode store;
    xmlnode node;
    char *key;

    if(hash == NULL || userid == NULL)
    {
	return -1;
    }

    key = j_strdup(jid_full(jid_user(userid)));
    store = xhash_get(hash, key);

    /* User not previously registered. Set up */
    if(store == NULL)
    {
	store = xmlnode_new_tag("users");
    }
    else
    {
        p = pool_new();
        node = xmlnode_get_tag(store, spools(p, "item?jid=", jid_full(userid), p)); 
	pool_free(p);

        /* If already in the node, ignore */
        if(node != NULL)
	    return 0;
    }

    if(details != NULL)
    {
	store = xmlnode_dup(details);
    }
    else if(userid->resource != NULL)
    {
        node = xmlnode_new_tag("item");
	xmlnode_put_attrib(node, "jid", jid_full(userid));
	
	xmlnode_insert_node(store, node);
    }

    xhash_put(hash, key, store);
    return 1;
}

int remove_affiliate(xht hash, jid userid)
{
    pool p;
    xmlnode store;
    xmlnode node;
    char *key;

    if(hash == NULL || userid == NULL)
    {
	return -1;
    }

    key = j_strdup(jid_full(jid_user(userid)));
    store = xhash_get(hash, key);

    if(store == NULL)
	return 1;

    p = pool_new();
    node = xmlnode_get_tag(store, spools(p, "item?jid=", jid_full(userid), p)); 
    pool_free(p);
    
    if(node == NULL)
        return 1;

    xmlnode_hide(node);

    xhash_put(hash, key, store);
    return 1;
}

xmlnode revoke_affiliate(cnr room, xht hash, jid userid)
{
    pool p;
    cnu user;
    jid jabberid;
    xmlnode store;
    xmlnode current;
    xmlnode result;
    char *userjid;
    char *key;

    if(hash == NULL || userid == NULL)
    {
	return NULL;
    }

    key = j_strdup(jid_full(jid_user(userid)));
    store = xhash_get(hash, key);

    if(store == NULL)
	return NULL;

    p = pool_new();

    current = xmlnode_get_tag(store, "item");

    if(current != NULL)
    {
	result = xmlnode_dup(store);

        for(current = xmlnode_get_firstchild(store); current != NULL; current = xmlnode_get_nextsibling(current))    
        {
	    userjid = xmlnode_get_attrib(current, "jid");

	    if(userjid != NULL)
	    {
	        jabberid = jid_new(p, userjid);
	        user = xhash_get(room->remote, jid_full(jabberid)); 

	        if(user != NULL)
	        {
		    update_presence(user);
	        }
	    }
	}
    }
    else
    {
	result = NULL;
    }

    xhash_zap(hash, key);
    pool_free(p);

    return result;
}

void change_affiliate(char *affiliation, cnu sender, jid user, char *reason, jid by)
{
    char *key;
    cnr room;
    cnu from;
    taffil current;
    xmlnode data, invite, x;

    if(affiliation == NULL || sender == NULL || user == NULL)
    {
	log_debug(NAME, "[%s] Missing attributes", FZONE);
	return;
    }

    room = sender->room;
    key = j_strdup(jid_full(user));
    current = affiliation_level(room, user);

    /* if not changing affiliation, just return */
    if(j_strcmp(current.msg, affiliation) == 0)
    {
	log_debug(NAME, "[%s] Affiliation not changing", FZONE);
	return;
    }

    /* Clear any old affiliation */
    if(j_strcmp(affiliation, "admin") != 0)
    {
	revoke_affiliate(room, room->admin, user);
    }

    if(j_strcmp(affiliation, "member") != 0)
    {
	revoke_affiliate(room, room->member, user);
    }

    if(j_strcmp(affiliation, "outcast") != 0)
    {
	revoke_affiliate(room, room->outcast, user);
    }

    /* Update to new affiliation */
    if(j_strcmp(affiliation, "admin") == 0)
    {
	add_affiliate(room->admin, user, NULL);
    }
    else if(j_strcmp(affiliation, "member") == 0)
    {
	add_affiliate(room->member, user, NULL);

	if(room->invitation == 1 && !in_room(room, user))
	{
            x = xmlnode_new_tag("x");
	    xmlnode_put_attrib(x, "xmlns", NS_MUC_USER);
	    invite = xmlnode_insert_tag(x, "invite");
	    xmlnode_put_attrib(invite, "to", jid_full(jid_user(user)));
	    xmlnode_insert_cdata(xmlnode_insert_tag(invite, "reason"), "Added as a member", -1);
	    con_room_send_invite(sender, x);
	}
    }
    else if(j_strcmp(affiliation, "outcast") == 0)
    {
	data = xmlnode_new_tag("reason");
	from = xhash_get(room->remote, jid_full(by));

	if(reason == NULL)
	{
	    xmlnode_insert_cdata(data, "None given", -1);
	}
	else
	{
	    xmlnode_insert_cdata(data, reason, -1);
	}

	if(from != NULL)
        {
	    xmlnode_put_attrib(data, "actor", jid_full(jid_user(from->realid)));
	    xmlnode_put_attrib(data, "nick", xmlnode_get_data(from->nick));
	}
	else
	{
	    xmlnode_put_attrib(data, "actor", jid_full(by));
	}

	add_affiliate(room->outcast, user, data);
    }

    return;
}

void add_role(xht hash, cnu user)
{
    char *key;
    key = j_strdup(jid_full(user->realid));
    log_debug(NAME, "[%s] About to add Moderator [%s]", FZONE, key);
    xhash_put(hash, key, (void*)user);
}

void revoke_role(xht hash, cnu user)
{
    char *key;
    key = j_strdup(jid_full(user->realid));
    xhash_zap(hash, key);
}

void change_role(char *role, cnu sender, jid user, char *reason)
{
    char *key, *result;
    cnr room;
    cnu data;
    jid userid;
    trole current;
    xmlnode node, userlist;

    log_debug(NAME, "[%s] Role change request - %s to %s", FZONE, jid_full(user), role);

    if(role == NULL || user == NULL)
    {
	log_debug(NAME, "[%s] Missing attributes", FZONE);
	return;
    }

    room = sender->room;
    key = j_strdup(jid_full(user));
    data = xhash_get(room->remote, key);

    if(data == NULL)
    {
	if(user->resource == NULL)
	{
	    userlist = get_roster(room, user);

	    if(userlist != NULL)
	    {
		for(node = xmlnode_get_firstchild(userlist); node != NULL; node = xmlnode_get_nextsibling(node))
		{
		    result = xmlnode_get_attrib(node, "jid");
		    userid = jid_new(xmlnode_pool(node), result);
		    change_role(role, sender, userid, reason);
		}
	    }
	    else
	    {
	        log_debug(NAME, "[%s] User not found", FZONE);
	    }
	    return;
	}
	else
	{
	    log_debug(NAME, "[%s] User not found", FZONE);
	    return;
	}
    }

    current = role_level(room, user);

    /* if not changing role, just return */
    if(j_strcmp(current.msg, role) == 0)
    {
	log_debug(NAME, "[%s] Role not changing", FZONE);
        update_presence(data);
	return;
    }

    /* Clear any old roles */
    if(j_strcmp(role, "moderator") != 0)
    {
	revoke_role(room->moderator, data);
    }

    if(j_strcmp(role, "participant") != 0)
    {
	revoke_role(room->participant, data);
    }

    /* Update to new role */
    if(j_strcmp(role, "moderator") == 0)
    {
	add_role(room->moderator, data);
	log_debug(NAME, "[%s] Added Moderator", FZONE);
    }
    else if(j_strcmp(role, "participant") == 0)
    {
	add_role(room->participant, data);
	log_debug(NAME, "[%s] Added Participant", FZONE);
    }
    else if(j_strcmp(role, "none") == 0)
    {
	if(reason == NULL)
	{
	    reason = j_strdup("None given");
	}
	log_debug(NAME, "[%s] Call kick routine with reason %s", FZONE, reason);

	data->leaving = 1;
	adm_user_kick(sender, data, reason);
	return;
    }

    update_presence(data);
    return;
}
